package net.pricefx.common.apibuilder;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public final class MethodInvocation {

	private MethodInvocation(String methodNameToken, String methodNameId,  Object ... arguments) {
		this.methodNameToken = methodNameToken;
		this.methodNameId = methodNameId;
		if (arguments == null || arguments.length == 0) 
			this.arguments = Collections.emptyList();
		else
			this.arguments = Collections.unmodifiableList(Arrays.asList(arguments));
	}
	
	private final String methodNameToken;
	private String methodNameId;
	private final List<Object> arguments;
	
	@Override public String toString() { return methodNameToken; }

	public String methodNameToken() { return methodNameToken; }
	public String methodNameId() { return methodNameId; }
	public List<Object> arguments() {return arguments; }
	
	public static MethodInvocation of(String methodNameToken, String methodNameId,  Object ... arguments) {
		return new MethodInvocation(methodNameToken, methodNameId, arguments);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((methodNameId == null) ? 0 : methodNameId.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		MethodInvocation other = (MethodInvocation) obj;
		if (methodNameId == null) {
			if (other.methodNameId != null) return false;
		} else if (!methodNameId.equals(other.methodNameId)) return false;
		return true;
	}
	
}