package net.pricefx.common.apibuilder.chart;

import java.math.BigDecimal;

import net.pricefx.common.api.chart.ChartType;
import net.pricefx.common.api.chart.ColorAxisType;
import net.pricefx.common.api.chart.LimitPosition;
import net.pricefx.common.api.chart.LimitType;
import net.pricefx.common.api.chart.SortType;
import net.pricefx.common.apibuilder.AbstractAdditionalMeasure;
import net.pricefx.common.apibuilder.AbstractBuilder;
import net.pricefx.common.apibuilder.AbstractOptions;
import net.pricefx.common.apibuilder.AbstractSeries;
import net.pricefx.common.apibuilder.BacklinkBuilder;
import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.apibuilder.MethodInvocation;
import net.pricefx.common.apibuilder.ObjectIdentity;

public class HeatmapChart extends AbstractBuilder {


	private HeatmapOptions options;
	private HeatmapSeries series;
	private Dictionary<HeatmapChart> dictionary;
	
	public HeatmapChart(String instanceId, String classId, ObjectIdentity parentChartObjectIdentity, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
		super(instanceId, classId, idGenerator, invocations);
	}
	
	public HeatmapSeries getSeries() {
		String id = addInvocation("getSeries");
		if (series == null) 
			series = new HeatmapSeries(this, id, "Series", idGenerator, invocations);
		return series;
	}
	
	public HeatmapOptions getOptions() {
		String id = addInvocation("getOptions");
		if (options == null) 
			options = new HeatmapOptions(this, id, "Options", idGenerator, invocations);
		return options;
	}
	
	public Dictionary<HeatmapChart> getDictionary() {
		String id = addInvocation("getDictionary");
		if (dictionary == null) 
			dictionary = new Dictionary<>(this, id, "Dictionary");
		return dictionary;
	}
	
	@Override
	public ChartBuilderResult build() {
		addInvocation(METHOD_BUILD);

		ChartBuilderResult r = new ChartBuilderResult();
		r.invocations = invocations.get();
		r.additionalInfo = ChartType.HEATMAP;
		return r;
	}
	
	public static class HeatmapSeries extends AbstractSeries<HeatmapChart,HeatmapSeries> {

		public HeatmapSeries(HeatmapChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}

		
		public HeatmapSeries setHideDataLabels(Boolean hideDataLabels) {

			addInvocation("setHideDataLabels",hideDataLabels);
			return this;
		}
		
		public LimitToMeasure<HeatmapSeries> setLimitTo(LimitPosition limitPosition, BigDecimal amount, LimitType limitType) {
			
			String id = addInvocation("setLimitTo",limitPosition, amount, limitType);
			return new LimitToMeasure<HeatmapSeries>(this, id, "LimitToMeasure");
		}
		
		public HeatmapSeries setAxisX(String axisX) {

			addInvocation("setAxisX",axisX);
			return this;
		}
		
		public HeatmapSeries withSortByAxisX(SortType sortType) {

			addInvocation("withSortByAxisX",sortType);
			return this;
		}
		
		public HeatmapSeries setAxisY(String axisY) {

			addInvocation("setAxisY",axisY);
			return this;
		}

		public HeatmapSeries withSortByAxisY(SortType sortType) {
			
			addInvocation("withSortByAxisY",sortType);
			return this;
		}
		
		public AggrFieldAxis<HeatmapSeries> setColorAxis(String colorAxis) {

			String id = addInvocation("setColorAxis",colorAxis);
			return new AggrFieldAxis<HeatmapSeries>(this, id, "ColorAxis");
		}

		public HeatmapSeries normalizeAcrossColumns() {
	         // "y":  normalize the values within the categories shown on the Y-axis (so that the sum in a row = 100%).aka 'normalization across columns'
		    addInvocation("normalizeAcrossColumns");
		    return this;
		}

		public HeatmapSeries normalizeAcrossRows() {
             // "x":  normalize the values within the categories shown on the X-axis (so that the sum in a column = 100%); aka 'normalization across rows'.
		    addInvocation("normalizeAcrossRows");
		    return this;
		}

		public HeatmapAdditionalMeasure<HeatmapSeries> addAdditionalMeasure() {
			
			String id = addInvocation("addAdditionalMeasure");
			return new HeatmapAdditionalMeasure<HeatmapSeries>(this, id, "AdditionalMeasure", idGenerator, invocations);
		}
		
		public Formula<HeatmapSeries> setFormula() {
			
			String id = addInvocation("setFormula");
			return new Formula<HeatmapSeries>(this, id, "Formula");
		}
		
		public HeatmapSeries setGeneratedQueryDto(String generatedQueryDto) {
			
			addInvocation("setGeneratedQueryDto",generatedQueryDto);
			return this;
		}

	}
	
	public static class HeatmapOptions extends AbstractOptions<HeatmapChart,HeatmapOptions> {

		public HeatmapOptions(HeatmapChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}
		
		public HeatmapOptions setColorAxisType(ColorAxisType colorAxisType) {

			addInvocation("setColorAxisType",colorAxisType);
			return this;
		}
		
		public HeatmapRangedColorAxis withRangedColorAxis() {

			String id = addInvocation("withRangedColorAxis");
			return new HeatmapRangedColorAxis(this, id, "RangedColorAxis", idGenerator, invocations);
		}
		
		public HeatmapInterpolatedColorAxis withInterpolatedColorAxis() {
			
			String id = addInvocation("withInterpolatedColorAxis");
			return new HeatmapInterpolatedColorAxis(this, id, "InterpolatedColorAxis", idGenerator, invocations);
		}

	}

	public static class HeatmapRangedColorAxis extends BacklinkBuilder<HeatmapOptions> {

		
		public HeatmapRangedColorAxis(HeatmapOptions backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink);
		}
		
		public HeatmapRangedColorAxis addRange(String color, BigDecimal from, BigDecimal to) {

			addInvocation("addRange",color, from, to);
			return this;
		}

	}
	
	public static class HeatmapInterpolatedColorAxis extends BacklinkBuilder<HeatmapOptions> {
		
		
		public HeatmapInterpolatedColorAxis(HeatmapOptions backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink);
		}
		
		public HeatmapInterpolatedColorAxis addStop(String color, Integer stop) {
			
			addInvocation("addStop",color, stop);
			return this;
		}
		
	}
		
	public static class HeatmapAdditionalMeasure<ATTACHEDTO extends ObjectIdentity> extends AbstractAdditionalMeasure<ATTACHEDTO,HeatmapAdditionalMeasure<ATTACHEDTO>> {

		public HeatmapAdditionalMeasure(ATTACHEDTO attachedTo, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, attachedTo, idGenerator, invocations);
		}

		
		public Expression<HeatmapAdditionalMeasure<ATTACHEDTO>> withExpression() { 
			
			String id = addInvocation("withExpression");
			return new Expression<HeatmapAdditionalMeasure<ATTACHEDTO>>(this, id, "Expression");
		}

	}
}
