package net.pricefx.formulaengine.optimization;

import net.pricefx.formulaengine.DatamartContext;

import java.util.Map;

public interface ModelFormulaContextReadOnly {
    /**
     * @return The unique name of this model
     */
    String uniqueName();

    /**
     * @return The label of this model
     */
    String label();

    /**
     * This gives access to the previous steps saved inputs.
     * <p>
     * Those inputs are organized per step and per tab in the step.
     * <p>
     * From a given step, it is not possible to access its inputs (for this
     * {@link net.pricefx.formulaengine.scripting.PublicGroovyAPI#input(String)}
     * or the {@code input} binding can be used) or the next step's inputs.
     *
     * @param stepName The name of the step as defined in the model class
     * @param tabName  The name of the tab as defined in the model class
     * @return The unmodifiable inputs of the tab in the step
     */
    Map<String, Object> inputs(String stepName, String tabName);

    /**
     * This gives access to calculations' results as returned by their formula.
     * <p>
     * Those outputs are organized per step and per calculation name in the step.
     * <p>
     * Only the formula-based calculations have results.
     * <p>
     * From a given step, it is not possible to access the next
     * steps' calculations' outputs.
     * <p>
     * From a given calculation in a sequence, it is not possible to access the next
     * calculations' outputs.
     *
     * @param stepName        The name of the step as defined in the model class
     * @param calculationName The name of the calculation as defined in the model class
     * @return The unmodifiable ordered results of the calculation returned by its formula
     */
    Map<String, Object> outputs(String stepName, String calculationName);

    /**
     * @param name The table name in this model
     * @return The existing model table or {@code null} if it doesn't exist
     */
    DatamartContext.Table table(String name);
}
